export function createValueCaster(argConfig) {
    if (argConfig.multiple) {
        return new ArrayValueCaster(argConfig);
    }
    return createSingleValueCaster(argConfig);
}
function createSingleValueCaster(argConfig) {
    switch (argConfig.type) {
        case 'string':
            return new StringValueCaster(argConfig);
        case 'number':
            return new NumberValueCaster(argConfig);
        case 'boolean':
            return new BooleanValueCaster(argConfig);
    }
}
class BaseValueCaster {
    constructor(config) {
        this.argConfig = config;
    }
    cast(value, isset) {
        if (!isset) {
            return this.argConfig.default;
        }
        if (value.length === 0) {
            return this.argConfig.const;
        }
        return undefined;
    }
}
class SingleValueCaster extends BaseValueCaster {
    cast(value, isset) {
        const baseCasted = super.cast(value, isset);
        if (baseCasted !== undefined) {
            return baseCasted;
        }
        return this.castInternal(this.getSingleValue(value));
    }
    getSingleValue(value) {
        var _a;
        return (_a = value[0]) !== null && _a !== void 0 ? _a : undefined;
    }
}
class StringValueCaster extends SingleValueCaster {
    castInternal(value) {
        return value !== undefined ? String(value) : undefined;
    }
}
class NumberValueCaster extends SingleValueCaster {
    castInternal(value) {
        return value !== undefined ? Number(value) : undefined;
    }
}
class BooleanValueCaster extends SingleValueCaster {
    castInternal(value) {
        return value !== undefined ? !['false', '0'].includes(value) : undefined;
    }
}
class ArrayValueCaster extends BaseValueCaster {
    constructor(config) {
        super(config);
        this.itemCaster = createSingleValueCaster(config);
    }
    cast(value, isset) {
        const baseCasted = super.cast(value, isset);
        if (baseCasted !== undefined && baseCasted.length > 0) {
            return baseCasted;
        }
        if (value.length === 0 && !isset && !this.argConfig.positional) {
            return undefined;
        }
        return value.map((v) => this.itemCaster.cast([v], isset));
    }
}
//# sourceMappingURL=cast.js.map