"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.createValueCaster = createValueCaster;
function createValueCaster(argConfig) {
    if (argConfig.multiple) {
        return new ArrayValueCaster(argConfig);
    }
    return createSingleValueCaster(argConfig);
}
function createSingleValueCaster(argConfig) {
    switch (argConfig.type) {
        case 'string':
            return new StringValueCaster(argConfig);
        case 'number':
            return new NumberValueCaster(argConfig);
        case 'boolean':
            return new BooleanValueCaster(argConfig);
    }
}
var BaseValueCaster = /** @class */ (function () {
    function BaseValueCaster(config) {
        this.argConfig = config;
    }
    BaseValueCaster.prototype.cast = function (value, isset) {
        if (!isset) {
            return this.argConfig.default;
        }
        if (value.length === 0) {
            return this.argConfig.const;
        }
        return undefined;
    };
    return BaseValueCaster;
}());
var SingleValueCaster = /** @class */ (function (_super) {
    __extends(SingleValueCaster, _super);
    function SingleValueCaster() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SingleValueCaster.prototype.cast = function (value, isset) {
        var baseCasted = _super.prototype.cast.call(this, value, isset);
        if (baseCasted !== undefined) {
            return baseCasted;
        }
        return this.castInternal(this.getSingleValue(value));
    };
    SingleValueCaster.prototype.getSingleValue = function (value) {
        var _a;
        return (_a = value[0]) !== null && _a !== void 0 ? _a : undefined;
    };
    return SingleValueCaster;
}(BaseValueCaster));
var StringValueCaster = /** @class */ (function (_super) {
    __extends(StringValueCaster, _super);
    function StringValueCaster() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    StringValueCaster.prototype.castInternal = function (value) {
        return value !== undefined ? String(value) : undefined;
    };
    return StringValueCaster;
}(SingleValueCaster));
var NumberValueCaster = /** @class */ (function (_super) {
    __extends(NumberValueCaster, _super);
    function NumberValueCaster() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NumberValueCaster.prototype.castInternal = function (value) {
        return value !== undefined ? Number(value) : undefined;
    };
    return NumberValueCaster;
}(SingleValueCaster));
var BooleanValueCaster = /** @class */ (function (_super) {
    __extends(BooleanValueCaster, _super);
    function BooleanValueCaster() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    BooleanValueCaster.prototype.castInternal = function (value) {
        return value !== undefined ? !['false', '0'].includes(value) : undefined;
    };
    return BooleanValueCaster;
}(SingleValueCaster));
var ArrayValueCaster = /** @class */ (function (_super) {
    __extends(ArrayValueCaster, _super);
    function ArrayValueCaster(config) {
        var _this = _super.call(this, config) || this;
        _this.itemCaster = createSingleValueCaster(config);
        return _this;
    }
    ArrayValueCaster.prototype.cast = function (value, isset) {
        var _this = this;
        var baseCasted = _super.prototype.cast.call(this, value, isset);
        if (baseCasted !== undefined && baseCasted.length > 0) {
            return baseCasted;
        }
        if (value.length === 0 && !isset && !this.argConfig.positional) {
            return undefined;
        }
        return value.map(function (v) { return _this.itemCaster.cast([v], isset); });
    };
    return ArrayValueCaster;
}(BaseValueCaster));
//# sourceMappingURL=cast.js.map