"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateArgConfig = validateArgConfig;
exports.checkEnoughPositionalValues = checkEnoughPositionalValues;
exports.checkAllPositionalValuesUsed = checkAllPositionalValuesUsed;
exports.createValueValidator = createValueValidator;
var exceptions_1 = require("../exceptions");
var utils_1 = require("./utils");
function validateArgConfig(config, usedArgs) {
    if (!config.name.startsWith('-') && config.required !== undefined) {
        throw new exceptions_1.ArgumentConfigError("Positional argument ".concat(config.name, " cannot be required."));
    }
    if (config.name.startsWith('-') && !config.name.startsWith('--')) {
        throw new exceptions_1.ArgumentConfigError("Argument name ".concat(config.name, " is invalid."));
    }
    if (config.alias !== undefined && (!config.alias.startsWith('-') || config.alias.startsWith('--'))) {
        throw new exceptions_1.ArgumentConfigError("Argument name ".concat(config.name, " is invalid."));
    }
    if (usedArgs.has(config.name)) {
        throw new exceptions_1.ArgumentConfigError("Argument ".concat(config.name, " already exists."));
    }
    if (config.alias !== undefined && usedArgs.has(config.alias)) {
        throw new exceptions_1.ArgumentConfigError("Argument ".concat(config.alias, " already exists."));
    }
}
function checkEnoughPositionalValues(values, argConfig) {
    var errorMessage = "Not enough positional arguments. ".concat(argConfig.name, " is required.");
    if (!argConfig.multiple) {
        if (!argConfig.allowEmpty && values.length === 0) {
            throw new exceptions_1.ArgumentValueError(errorMessage);
        }
        return;
    }
    if (!argConfig.allowEmpty && values.length === 0) {
        throw new exceptions_1.ArgumentValueError(errorMessage);
    }
    if (!argConfig.allowEmpty && argConfig.valuesCount !== undefined && argConfig.valuesCount > values.length) {
        throw new exceptions_1.ArgumentValueError(errorMessage);
    }
}
function checkAllPositionalValuesUsed(values) {
    if (values.length > 0) {
        throw new exceptions_1.ArgumentValueError('Too many positional arguments.');
    }
}
function createValueValidator(argConfig) {
    if (argConfig.multiple) {
        return new ArrayValueValidator(argConfig);
    }
    return createSingleValueValidator(argConfig);
}
function createSingleValueValidator(argConfig) {
    switch (argConfig.type) {
        case 'string':
            return new StringValueValidator(argConfig);
        case 'number':
            return new NumberValueValidator(argConfig);
        case 'boolean':
            return new BooleanValueValidator(argConfig);
    }
}
var BaseValueValidator = /** @class */ (function () {
    function BaseValueValidator(argConfig) {
        this.argConfig = argConfig;
    }
    BaseValueValidator.prototype.validateBeforeCast = function (value, isset) {
        if (!isset && this.argConfig.required && value.length === 0) {
            throw new exceptions_1.ArgumentValueError("Argument ".concat((0, utils_1.formatArgNameWithAlias)(this.argConfig), " is required."));
        }
        if (isset && !this.argConfig.allowEmpty && value.length === 0) {
            throw new exceptions_1.ArgumentValueError("Argument ".concat((0, utils_1.formatArgNameWithAlias)(this.argConfig), " cannot be empty."));
        }
    };
    ;
    BaseValueValidator.prototype.validateAfterCast = function (value) {
        if (this.argConfig.validator !== undefined && !this.argConfig.validator(value)) {
            throw new exceptions_1.ArgumentValueError("Argument ".concat((0, utils_1.formatArgNameWithAlias)(this.argConfig), " value is invalid."));
        }
    };
    return BaseValueValidator;
}());
var SingleValueValidator = /** @class */ (function (_super) {
    __extends(SingleValueValidator, _super);
    function SingleValueValidator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SingleValueValidator.prototype.validateBeforeCast = function (value, isset) {
        _super.prototype.validateBeforeCast.call(this, value, isset);
        if (value.length > 1) {
            throw new exceptions_1.ArgumentValueError("Argument ".concat((0, utils_1.formatArgNameWithAlias)(this.argConfig), " expects a single value."));
        }
        if (this.argConfig.choices !== undefined && !this.argConfig.choices.includes(value[0])) {
            throw new exceptions_1.ArgumentValueError("Argument ".concat((0, utils_1.formatArgNameWithAlias)(this.argConfig), " value must be one of ").concat(this.argConfig.choices.join(', '), "."));
        }
    };
    return SingleValueValidator;
}(BaseValueValidator));
var StringValueValidator = /** @class */ (function (_super) {
    __extends(StringValueValidator, _super);
    function StringValueValidator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return StringValueValidator;
}(SingleValueValidator));
var NumberValueValidator = /** @class */ (function (_super) {
    __extends(NumberValueValidator, _super);
    function NumberValueValidator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    NumberValueValidator.prototype.validateBeforeCast = function (value, isset) {
        _super.prototype.validateBeforeCast.call(this, value, isset);
        if (value.length > 0 && isNaN(parseFloat(value[0]))) {
            throw new exceptions_1.ArgumentValueError("Argument ".concat((0, utils_1.formatArgNameWithAlias)(this.argConfig), " value is not a number."));
        }
    };
    return NumberValueValidator;
}(SingleValueValidator));
var BooleanValueValidator = /** @class */ (function (_super) {
    __extends(BooleanValueValidator, _super);
    function BooleanValueValidator() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    BooleanValueValidator.prototype.validateBeforeCast = function (value, isset) {
        _super.prototype.validateBeforeCast.call(this, value, isset);
        if (value.length > 0 && !['true', 'false', '1', '0'].includes(value[0].toLowerCase())) {
            throw new exceptions_1.ArgumentValueError("Argument ".concat((0, utils_1.formatArgNameWithAlias)(this.argConfig), " value is not a boolean."));
        }
    };
    return BooleanValueValidator;
}(SingleValueValidator));
var ArrayValueValidator = /** @class */ (function (_super) {
    __extends(ArrayValueValidator, _super);
    function ArrayValueValidator(argConfig) {
        var _this = _super.call(this, argConfig) || this;
        _this.itemValidator = createSingleValueValidator(_this.argConfig);
        return _this;
    }
    ArrayValueValidator.prototype.validateBeforeCast = function (value, isset) {
        var _this = this;
        _super.prototype.validateBeforeCast.call(this, value, isset);
        (value !== null && value !== void 0 ? value : []).forEach(function (v) { return _this.itemValidator.validateBeforeCast([v], isset); });
    };
    ArrayValueValidator.prototype.validateAfterCast = function (value) {
        var _this = this;
        _super.prototype.validateAfterCast.call(this, value);
        (value !== null && value !== void 0 ? value : []).forEach(function (v) { return _this.itemValidator.validateAfterCast([v]); });
    };
    return ArrayValueValidator;
}(BaseValueValidator));
//# sourceMappingURL=validation.js.map